#include <RedFly.h>
#include <Servo.h>

Servo myservo;
int pos = 0;

int ledPin = 11;                // LED liegt am (digitalen) Pin 11.

byte ip[]      = { 192, 168,  2, 1 }; //ip from shield (server)
byte netmask[] = { 255,255,255,  0 }; //netmask

#define UDP_PORT  (80) //local UDP port on the shield
uint8_t hUDP=0xFF;  //socket handles

//serial format: 9600 Baud, 8N2
void debugout(char *s)  { RedFly.disable(); Serial.print(s);   RedFly.enable(); }
void debugoutln(char *s){ RedFly.disable(); Serial.println(s); RedFly.enable(); }

void setup()
{
  pinMode(ledPin, OUTPUT); //Setzt den ledPin als Ausgang
  uint8_t ret;
  
  //init the WiFi module on the shield
  ret = RedFly.init();
  if(ret)
  {
    debugoutln("INIT ERR"); //there are problems with the communication between the Arduino and the RedFly
  }
  else
  {
    //scan for wireless networks (must be run before join command)
    RedFly.scan();

    //join network
    ret = RedFly.join("arduino", "12345678", INFRASTRUCTURE);
    if(ret)
    {
      debugoutln("JOIN ERR");
      for(;;); //do nothing forevermore
    }
    else
    {
      //set ip config
      ret = RedFly.begin(ip, 0, 0, netmask);
      if(ret)
      {
        debugoutln("BEGIN ERR");
        RedFly.disconnect();
        for(;;); //do nothing forevermore
      }
    }
  }
  Serial.begin(9600); // open the serial connection at 9600bps
}

void loop()
{
  uint8_t sock;
  uint16_t buf_len, rd, len;
  uint16_t port; //incomming UDP port
  uint8_t ip[4]; //incomming UDP ip
  
  char buffer[3], *ptr;
  int receiveVar1;

  //check if sockets are opened
  if(RedFly.socketClosed(hUDP) || (hUDP == 0xFF))
  {
    hUDP = RedFly.socketListen(PROTO_UDP, UDP_PORT); //open UDP socket
  }

  //get data
  sock    = 0xFF; //0xFF = return data from all open sockets
  ptr     = &buffer[0];
  buf_len = 0;
  
  do
  {
    rd = RedFly.socketRead(&sock, &len, ip, &port, (uint8_t*)ptr, sizeof(buffer)-buf_len);
    if((rd != 0) && (rd != 0xFFFF)) //0xFFFF = connection closed
    {    
      ptr     += rd;
      buf_len += rd;
    }
  }while(len != 0);
  
  //Buffer auswerten: wenn Länge gleich 3 und der Socket nicht geschlossen
  if(buf_len == 3 && (sock != 0xFF) && sock == hUDP)
  {
    //Pointer an Stelle 0 des Buffers
    ptr = &buffer[0];
    
    //erstes Byte des Buffers ist der Modus
    char modus = *(ptr);
    //zweites Byte des Buffers ist der jeweilige Pin
    int pin = (int)*(ptr + 1);
    //drittes Byte des Buffers ist der Zustand
    int state = (int)*(ptr + 2);
    
    //Wenn der Pin zwischen 4 und 19 liegt
    if(pin>=4 && pin<=19)
    {
      switch(pin){
        case 14: pin = A0; break;
        case 15: pin = A1; break;
        case 16: pin = A2; break;
        case 17: pin = A3; break;
        case 18: pin = A4; break;
        case 19: pin = A5; break;
    }          
    if (*ptr == 'D') 
    { //digital output
      //Pin als Output definieren
      pinMode(pin, OUTPUT);
      //Schreiben des digitalen Wertes je nach empfangenen Zustand
      if(state == 0)
      {
        digitalWrite(pin, LOW);
      }else if(state == 1)
      {
        digitalWrite(pin, HIGH);
      }else
      {
        Serial.print("Wrong Command (not available state): ");
      }
    }else if (*ptr == 'A') 
    { //analog output
      //Wenn Zustand zwischen 0 und 100 liegt (Prozent)
      if(state>=0 && state<=100)
      {
        //Pin als Output definieren
        pinMode(pin, OUTPUT);
        //Schreiben des analogen Wertes (0 bis 255 max)
        //Umrechnung 255 gleich 100 Prozent
        analogWrite(pin, map(state, 0, 100, 0, 255));
      }else
      {
        Serial.print("Wrong Command (not available state): ");
      }
    }else if (*ptr == 'S')
    { //servo output
      //Wenn Zustand zwischen 0 und 100 liegt (Prozent)
      if(state>=0 && state<=100)
      {
        //Pin als Output definieren
        pinMode(pin, OUTPUT);
        //Nutzen der Servo Library und erzeugen des PWM-Signals an dem spezifischen Pin
        //Auslenkung max. 180
        //Umrechnung 180 gleich 100 Prozent
        myservo.attach(pin);
        myservo.write(map(state, 0, 100, 0, 180));
      }else
      {
        Serial.print("Wrong Command (not available state): ");
      }
    }else if (*ptr == 'B')
    { //read analog
      //Pin als Input definieren
      pinMode(pin, INPUT);
      uint8_t val[1];
      //Wert am Pin lesen und in die Variable speichern
      val[0] = map(analogRead(pin), 0, 1023, 0, 100);
      delay(100); 
      //Senden des Wertes an die IP, von dem der Eingangs-Buffer empfangen wurde
      RedFly.socketSend(hUDP, val, 1, ip, UDP_PORT);
    }else if (*ptr == 'E')
    { //read digital
      //Pin als Input definieren
      pinMode(pin, INPUT);
      uint8_t val[1];
      //Wert am Pin lesen und in die Variable speichern
      if(digitalRead(pin)==0)
      {
        val[0] = 0;
      }else
      {
        val[0] = 1;
      }
      delay(100);
      //Senden des Wertes an die IP und an den Port, von dem der Eingangs-Buffer empfangen wurde
      RedFly.socketSend(hUDP, val, 1, ip, UDP_PORT);
    }else
    {
      Serial.print("Wrong Command (not available modus): ");
    }
    }else
    {
      Serial.print("Wrong Command (not available pin): ");
    }
    //Ausgeben des Eingangsbuffers über den Serial Monitor
    Serial.print(modus);
    Serial.print("-");
    Serial.print(pin);
    Serial.print("-");
    Serial.println(state);
  }
}
